import { __decorate } from "tslib";
import { Injectable } from '@angular/core';
import { Validators } from '@angular/forms';
export let FormServiceDynamic = class FormServiceDynamic {
    constructor(formBuilder) {
        this.formBuilder = formBuilder;
    }
    createForm(model, options) {
        const formGroup = {};
        for (const field in model) {
            if (model.hasOwnProperty(field)) {
                const fieldConfig = options && options[field] ? options[field] : {};
                formGroup[field] = this.createControl(model[field], fieldConfig);
            }
        }
        return this.formBuilder.group(formGroup);
    }
    createControl(value, config) {
        const formControl = this.formBuilder.control({ value: value, disabled: !config.enabled }, this.normalizeValidations(config.validations), this.normalizeAsyncValidations(config.asyncValidations));
        if (config.customValidation) {
            formControl.setValidators([config.customValidation]);
        }
        if (config.asyncCustomValidation) {
            formControl.setAsyncValidators([config.asyncCustomValidation]);
        }
        if (config.requiredIf) {
            formControl.setValidators([this.conditionallyRequiredValidator(config)]);
        }
        return formControl;
    }
    conditionallyRequiredValidator(config) {
        return (control) => {
            if (this.isConditionallyRequired(config)) {
                if (Validators.required(control)) {
                    return null;
                }
                else {
                    return { required: true };
                }
            }
            return null;
        };
    }
    isConditionallyRequired(config) {
        return config.requiredIf && config.requiredIf.condition ? config.requiredIf.condition() : false;
    }
    normalizeValidations(validations) {
        if (!validations) {
            return null;
        }
        const validatorArray = [];
        for (const key in validations) {
            if (validations.hasOwnProperty(key) && Validators.hasOwnProperty(key)) {
                validatorArray.push(Validators[key](validations[key]));
            }
        }
        return validatorArray.length > 0 ? Validators.compose(validatorArray) : null;
    }
    normalizeAsyncValidations(asyncValidations) {
        if (!asyncValidations) {
            return null;
        }
        const asyncValidatorArray = [];
        for (const key in asyncValidations) {
            if (asyncValidations.hasOwnProperty(key) && Validators.hasOwnProperty(key)) {
                asyncValidatorArray.push(Validators[key](asyncValidations[key]));
            }
        }
        return asyncValidatorArray.length > 0 ? Validators.composeAsync(asyncValidatorArray) : null;
    }
};
FormServiceDynamic = __decorate([
    Injectable({
        providedIn: 'root'
    })
], FormServiceDynamic);
//# sourceMappingURL=form-service-dynamic.service.js.map